const WebSocket = require('ws');
const url = require('url');
const querystring = require('querystring');
const crypto = require('crypto');
const fs = require('fs');
const STATUS_FIRST_FRAME = 0;
const STATUS_CONTINUE_FRAME = 1;
const STATUS_LAST_FRAME = 2;
class WsParam {
  constructor(host, APPID, APIKey, APISecret, Text) {
    this.Host = host
    this.APPID = APPID;
    this.APIKey = APIKey;
    this.APISecret = APISecret;
    this.Text = Text;
    this.CommonArgs = { app_id: this.APPID };
    this.BusinessArgs = { aue: 'raw', auf: 'audio/L16;rate=16000', vcn: '********************************', tte: 'utf8' };
    this.Data = { status: 2, text: Buffer.from(this.Text).toString('base64') };
  }
  createUrl() {
    const date = new Date().toUTCString();
    const signatureOrigin = `host: ${this.Host}
date: ${date}
GET /v2/tts HTTP/1.1`;
    const signatureSha = crypto.createHmac('sha256', this.APISecret).update(signatureOrigin).digest('base64');
    const authorizationOrigin = `api_key="${this.APIKey}", algorithm="hmac-sha256", headers="host date request-line", signature="${signatureSha}"`;
    const authorization = Buffer.from(authorizationOrigin).toString('base64');
    const v = {
      authorization,
      date,
      host: this.Host
    };
    const websocketUrl = `wss://${this.Host}/v2/tts?${querystring.stringify(v)}`;
    return websocketUrl
  }
}
function onMessage(message) {
  try {
    const messageObj = JSON.parse(message);
    const code = messageObj.code;
    const sid = messageObj.sid;
    const audio = messageObj.data.audio;
    const audioBase64 = Buffer.from(audio, 'base64');
    const status = messageObj.data.status;
    console.log(messageObj);
    if (status === STATUS_LAST_FRAME) {
      console.log("ws is closed");
      ws.close();
    }
    if (code !== 0) {
      const errMsg = messageObj.message;
      console.log(`sid:${sid} call error:${errMsg} code is:${code}`);
    } else {
      fs.appendFileSync('********************************', audioBase64);
    }
  } catch (e) {
    console.log("receive msg,but parse exception:", e);
  }
}
function onError(error) {
  console.log("### error:", error);
}
function onClose() {
  console.log("### closed ###");
}
function onOpen() {
  const d = {
    common: wsParam.CommonArgs,
    business: wsParam.BusinessArgs,
    data: wsParam.Data,
  };
  const dStr = JSON.stringify(d);
  console.log("------>Start sending data");
  ws.send(dStr);
  if (fs.existsSync('********************************')) {
    fs.unlinkSync('********************************');
  }
}

const host = 'tts-api-sg.xf-yun.com'
const APPID = '********'
const APIKey = '********************************'
const APISecret = '********************************'
const wsParam = new WsParam(host, APPID, APIKey, APISecret, 'อากาศดีมากในวันนี้');
const websocketUrl = wsParam.createUrl();
const ws = new WebSocket(websocketUrl, { perMessageDeflate: false });
ws.on('message', onMessage);
ws.on('error', onError);
ws.on('close', onClose);
ws.on('open', onOpen);
